function mainGuided()
%mainGuided Solve LTL planning problem using automaton-guided approach
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

close all; clc;
yalmip('clear');

%% Set parameters
initParams;
fprintf('Number of stages per segment: %i.\n',N);

%% Create the environment
ex = 3;
env = envLTL(ex);
obstacles = env.work.unsafe;
plotEnv(env, obstacles);
ops.dim = env.util.dim;


%% Parse LTL formula and create non-deterministic Buchi automaton
formula = '[] safe && <> a && <> b && <> c';
path_to_ltl2ba = '/Users/ewolff/Software/ltl2ba-1.1/ltl2ba';
nba = getNba(formula, env, path_to_ltl2ba);

% Adjust the weights
nba.Aw = nba.A;
ind = find(nba.Aw(nba.acc,:));
for i = 1:length(ind)
    if ind(i) ~= nba.init
        nba.Aw(nba.acc, ind(i)) = 4; % Almost always infeasible trans
    end
end

%% Solve LTL planning problem
fprintf('Starting search...\n');
assert( 1 == length(nba.acc) );
pathSet = java.util.HashSet;
count = 0;
numPath = 0;
maxPath = 10;

tic;
solverTime = 0;
while numPath < maxPath && count < 50
    count = count + 1;

    % Create abstract path for suffix
    src = nba.acc;
    dst = nba.acc;
    [labelsSuf, pathSuf] = abstractPath(nba, src, dst,'cycle');
    pathSuf = pathSuf(1:end-1);

    % Create abstract path for prefix
    src = nba.init;
    dst = pathSuf;
    [labelsPre, pathPre] = abstractPath(nba, src, dst,'to_cycle');
    assert( length(pathPre) > 0 );

    % Rotate suffix to algin with prefix
    ind = find(pathSuf == pathPre(end), 1) - 1;
    pathSuf = circshift(pathSuf, [0,-ind]);
    pathSuf = [pathSuf, pathSuf(1)];
    labelsSuf = circshift(labelsSuf, [-ind,0]);

    fprintf('\npre: %s\n', num2str(pathPre));
    fprintf('suf: %s\n',   num2str(pathSuf));

    if ~pathSet.contains(strcat('pre: ', num2str(pathPre), '  suf: ', num2str(pathSuf)))
        numPath = numPath + 1;
        pathSet.add(strcat('pre: ', num2str(pathPre), '  suf: ', num2str(pathSuf)));

        [sol, traj, sys] = trajSim(labelsPre, labelsSuf, N, dt, model, options,...
                                   bigM, ops, blkVar);

        solverTime = solverTime + sol.solvertime;

        if ~isempty(find(traj,1)) || sol.problem == 0
            break
        end
    end

    % Update weights for computation of next shortest path.
    for i = 1:length(pathPre)-1
        nba.Aw(pathPre(i), pathPre(i+1)) = nba.Aw(pathPre(i), pathPre(i+1)) + 0.5;
    end
    for i = 1:length(pathSuf)-1
        nba.Aw(pathSuf(i), pathSuf(i+1)) = nba.Aw(pathSuf(i), pathSuf(i+1)) + 1;
    end
    nba.Aw(pathSuf(i+1), pathSuf(1)) = nba.Aw(pathSuf(i+1), pathSuf(1)) + 1;
    fprintf('\n###################################################\n');
end
if numPath == maxPath
    fprintf('Path limit of %i reached.  Finishing.\n', maxPath);
end
toc
fprintf('\nSolver time: %4.4f.\n', solverTime);
    
end
