function [traj, cost, sol] = getPrefix(Xpre, U, J, F_G, F_F, F_sys, F_init, ...
                                       nx, Npre, Nsuf, suffix, options)
%getPrefix Create prefix for sequential planner
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

% Create the prefix that satisfies safety specs and connects to the cycle.
tol = 1e-3;
z_connect = binvar(Nsuf, 1);
prefix_N = Xpre((Npre-1)*nx + 1:end);    % final state on prefix
F_connect = [prefix_N <= reshape(suffix(1:Nsuf*nx), nx, Nsuf)*z_connect + tol, ...
             prefix_N >= reshape(suffix(1:Nsuf*nx), nx, Nsuf)*z_connect - tol, ...
             sum(z_connect) == 1];

F = [F_G, F_F, F_sys, F_init, F_connect];

sol = solvesdp(F, J, options);
ctrl = double(U);
traj = double(Xpre);
cost = double(J);

end

