function [F, z] = safe(G, C, nx, X, N, bigM, blkSafe)
% safe Creates safety constraints
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

fprintf('Using bigM encoding...\n');

if ~isempty(G)
    % Create safety constraints
    nPoly = length(G);

    H = cell(1, nPoly);
    K = cell(1, nPoly);
    M = cell(1, nPoly);
    for i = 1:nPoly
        [h, k] = double(G(i));
        H{i} = h;
        K{i} = k;
        M{i} = bigM*ones(length(k), 1);
    end

    % TODO: vectorize to eliminate 'for' loop
    H = blkdiag(H{:});
    K = vertcat(K{:});
    M = blkdiag(M{:});
    I = kron(ones(nPoly, 1), C);

    assert( blkSafe >= 1 )   
    if blkSafe >= 2
        fprintf('Blocking safe by %i.\n', blkSafe);
    end

    F = [];
    for t = 1:N+1
        if mod(t + blkSafe - 1, blkSafe) == 0
            z = binvar(nPoly, 1);
            F = [F, sum(z) == 1];
        end
        F = [F, H*I*X((t-1)*nx+1: t*nx) <= K + M*(1 - z) ];
    end
else
    F = [];
end

end
